/***************************************************************************/
/* Copyright (C) 2006 Electronic Theatre Controls, Inc.                    */
/* All rights reserved.                                                    */
/*                                                                         */
/* This software is provided by Electronic Theatre Controls, Inc. ("ETC")  */
/* at no charge to the user to facilitate the use and implementation of    */
/* ANSI BSR E1.31-2006 BSR E1.31, Entertainment Technology - DMX512-A      */
/* Streaming Protocol (DSP).                                               */
/*                                                                         */
/* Redistribution and use in source and binary forms, with or without      */
/* modification, are permitted provided that the following conditions are  */
/* met:                                                                    */
/*                                                                         */
/*      Redistributions of source code, whether in binary form or          */
/*      otherwise must retain the above copyright notice, a list of        */
/*      modifications made, if any, a description of modifications made,   */
/*      this list of conditions and the following disclaimer.              */
/*                                                                         */
/* THIS SOFTWARE IS PROVIDED "AS IS" BY THE COPYRIGHT HOLDER AND WITHOUT   */
/* WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING, WITHOUT     */
/* LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,            */
/* MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.  THE     */
/* ENTIRE RISK AS TO THE QUALITY, OPERATION, AND PERFORMANCE OF THE        */
/* SOFTWARE IS WITH THE USER.                                              */
/*                                                                         */
/* UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY, WHETHER TORT          */
/* (INCLUDING NEGLIGENCE), CONTRACT OR OTHERWISE, SHALL ETC BE LIABLE TO   */
/* ANY PERSON OR ENTITY FOR ANY INDIRECT, SPECIAL, INCIDENTAL, OR          */
/* CONSEQUENTIAL DAMAGES OF ANY KIND INCLUDING WITHOUT LIMITATION, DAMAGES */
/* FOR LOSS OF GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR MALFUNCTION,   */
/* OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF SUCH         */
/* PARTY SHALL HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH DAMAGES.      */
/* THIS LIMITATION OF LIABILITY SHALL NOT APPLY (I) TO DEATH OR PERSONAL   */
/* INJURY RESULTING FROM SUCH PARTY'S NEGLIGENCE TO THE EXTENT APPLICABLE  */
/* LAW PROHIBITS SUCH LIMITATION, (II) FOR CONSUMERS WITH RESIDENCE IN A   */
/* MEMBER COUNTRY OF THE EUROPEAN UNION (A) TO DEATH OR PERSONAL INJURY    */
/* RESULTING FROM ETC'S NEGLIGENCE, (B) TO THE VIOLATION OF ESSENTIAL      */
/* CONTRACTUAL DUTIES BY ETC, AND (C) TO DAMAGES RESULTING FROM            */
/* ETC'S GROSS NEGLIGENT OR WILLFUL BEHAVIOR, SOME JURISDICTIONS DO NOT    */
/* ALLOW THE EXCLUSION OR LIMITATION OF INCIDENTAL OR CONSEQUENTIAL        */
/* DAMAGES, SO THIS EXCLUSION AND LIMITATION MAY NOT APPLY TO YOU.         */
/*                                                                         */
/* Neither the name of ETC, its affiliates, nor the names of its           */
/* contributors may be used to endorse or promote products derived from    */
/* this software without express prior written permission of the President */
/* of ETC.                                                                 */
/*                                                                         */
/* The user of this software agrees to indemnify and hold ETC, its         */
/* affiliates, and contributors harmless from and against any and all      */
/* claims, damages, and suits that are in any way connected to or arising  */ 
/* out of, either in whole or in part, this software or the user's use of  */
/* this software.                                                          */
/*                                                                         */
/* Except for the limited license granted herein, all right, title and     */
/* interest to and in the Software remains forever with ETC.               */
/***************************************************************************/

// CID.cpp: implementation of the CID class.
//
//////////////////////////////////////////////////////////////////////

#include "deftypes.h"
#include <string.h>
#include "CID.h"
#include <stdio.h>

////////////////////////////////////////////////////////////////////////////////////////
// Non-static functions
CID::CID()
{
	memset(m_cid, 0, CIDBYTES);
}

CID::~CID()
{

}

CID::CID(const uint1* pCID)
{
	memcpy(m_cid, pCID, CIDBYTES);
}


CID::CID(const CID& cid)
{
	memcpy(m_cid, cid.m_cid, CIDBYTES);
}

CID& CID::operator=(const CID& cid)
{
	memcpy(m_cid, cid.m_cid, CIDBYTES);
	return (*this);
}

void CID::Pack(uint1* pbuf) const
{
	memcpy(pbuf, m_cid, CIDBYTES);
}

void CID::Unpack(const uint1* pbuf)
{
	memcpy(m_cid, pbuf, CIDBYTES);
}

bool operator<(const CID& c1, const CID& c2) 
{
	return 0 > memcmp(c1.m_cid, c2.m_cid, CID::CIDBYTES);
}

bool operator==(const CID& c1, const CID& c2) 
{
	return 0 == memcmp(c1.m_cid, c2.m_cid, CID::CIDBYTES);
}

bool operator!=(const CID& c1, const CID& c2) 
{
	return 0 != memcmp(c1.m_cid, c2.m_cid, CID::CIDBYTES);
}


DCID::DCID(const uint1* pDCID):m_cid(pDCID) {}
DCID::DCID(const DCID& dcid):m_cid(dcid.m_cid) {}
DCID::DCID():m_cid() {}
DCID::~DCID() {}
void DCID::Pack(uint1* pbuf) const {m_cid.Pack(pbuf);}
void DCID::Unpack(const uint1* pbuf) {m_cid.Unpack(pbuf);}
DCID& DCID::operator=(const DCID& dcid) {m_cid = dcid.m_cid; return *this;}
bool operator<(const DCID& d1, const DCID& d2) {return d1.m_cid < d2.m_cid;}
bool operator==(const DCID& d1, const DCID& d2){return d1.m_cid == d2.m_cid;}
bool operator!=(const DCID& d1, const DCID& d2){return d1.m_cid != d2.m_cid;}


///////////////////////////////////////////////////////////////
// Static functions
//A quick helper function that puts fills a UUID string
static void UUIDFillString(uint1* pbuffer, const char* ptext)
{
	//This function assumes a lot of safety issues...
	//We'll at least only put in CIDBYTES bytes
	const char *pt = ptext;
	uint1 *pb = pbuffer;
	bool first = true; //Whether we are doing the first or second nibble of the byte
	
	while((*pt != 0) && (pb - pbuffer < CID::CIDBYTES))
	{
		uint1 offset = 0;
		if((*pt >= 0x30) && (*pt <= 0x39))
			offset = 0x30;
		else if((*pt >= 0x41) && (*pt <= 0x46))
			offset = 0x37;  //0x41 - 0x37 = 0xa
		else if((*pt >= 0x61) && (*pt <= 0x66))
			offset = 0x57;  //0x61 - 0x57 = 0xa
		
		if(offset != 0)
		{
			if(first)
			{
				*pb = static_cast<uint1>(*pt - offset);
				*pb <<= 4;
				first = false;
			}
			else
			{
				*pb |= *pt - offset;
				++pb;
				first = true;
			}
		}
		++pt;
	}
}

CID CID::StringToCID(const char* ptext)  
{
	uint1 pbuffer [CIDBYTES];  //Temporarily holds the cid string
	memset(pbuffer, 0, CIDBYTES);  //So if the string is invalid, a CID() is created
	UUIDFillString(pbuffer, ptext);
	return CID(pbuffer);
}

//Translates a cid to a preallocated text string of 37 bytes (includes terminating NULL0
void CID::CIDIntoString(const CID& cid, char* ptxt)
{
	sprintf(ptxt, "%02X%02X%02X%02X-%02X%02X-%02X%02X-%02X%02X-%02X%02X%02X%02X%02X%02X",
			cid.m_cid[0], cid.m_cid[1], cid.m_cid[2], cid.m_cid[3], cid.m_cid[4],
			cid.m_cid[5], cid.m_cid[6],	cid.m_cid[7], cid.m_cid[8],	cid.m_cid[9],
			cid.m_cid[10], cid.m_cid[11], cid.m_cid[12], cid.m_cid[13], cid.m_cid[14],
			cid.m_cid[15]);
}

DCID DCID::StringToDCID(const char* ptext)  
{
	uint1 pbuffer [DCIDBYTES];  //Temporarily holds the cid string
	memset(pbuffer, 0, DCIDBYTES);  //So if the string is invalid, a DCID() is created
	UUIDFillString(pbuffer, ptext);
	return DCID(pbuffer);
}

//Translates a cid to a preallocated text string of DCIDSTRINGBYTES bytes (includes terminating NULL)
void DCID::DCIDIntoString(const DCID& dcid, char* ptxt)
{
	CID::CIDIntoString(dcid.m_cid, ptxt);
}

//Translates a dcid as a tftp-able filename to a preallocated text string of DCIDFILEBYTES bytes, including the terminating NULL
void DCID::DCIDIntoFileName(const DCID& dcid, char* ptxt)
{
	CID::CIDIntoString(dcid.m_cid, ptxt);
	strcat(ptxt, ".ddl");
}

