/***************************************************************************/
/* Copyright (C) 2006 Electronic Theatre Controls, Inc.                    */
/* All rights reserved.                                                    */
/*                                                                         */
/* This software is provided by Electronic Theatre Controls, Inc. ("ETC")  */
/* at no charge to the user to facilitate the use and implementation of    */
/* ANSI BSR E1.31-2006 BSR E1.31, Entertainment Technology - DMX512-A      */
/* Streaming Protocol (DSP).                                               */
/*                                                                         */
/* Redistribution and use in source and binary forms, with or without      */
/* modification, are permitted provided that the following conditions are  */
/* met:                                                                    */
/*                                                                         */
/*      Redistributions of source code, whether in binary form or          */
/*      otherwise must retain the above copyright notice, a list of        */
/*      modifications made, if any, a description of modifications made,   */
/*      this list of conditions and the following disclaimer.              */
/*                                                                         */
/* THIS SOFTWARE IS PROVIDED "AS IS" BY THE COPYRIGHT HOLDER AND WITHOUT   */
/* WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING, WITHOUT     */
/* LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,            */
/* MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.  THE     */
/* ENTIRE RISK AS TO THE QUALITY, OPERATION, AND PERFORMANCE OF THE        */
/* SOFTWARE IS WITH THE USER.                                              */
/*                                                                         */
/* UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY, WHETHER TORT          */
/* (INCLUDING NEGLIGENCE), CONTRACT OR OTHERWISE, SHALL ETC BE LIABLE TO   */
/* ANY PERSON OR ENTITY FOR ANY INDIRECT, SPECIAL, INCIDENTAL, OR          */
/* CONSEQUENTIAL DAMAGES OF ANY KIND INCLUDING WITHOUT LIMITATION, DAMAGES */
/* FOR LOSS OF GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR MALFUNCTION,   */
/* OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF SUCH         */
/* PARTY SHALL HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH DAMAGES.      */
/* THIS LIMITATION OF LIABILITY SHALL NOT APPLY (I) TO DEATH OR PERSONAL   */
/* INJURY RESULTING FROM SUCH PARTY'S NEGLIGENCE TO THE EXTENT APPLICABLE  */
/* LAW PROHIBITS SUCH LIMITATION, (II) FOR CONSUMERS WITH RESIDENCE IN A   */
/* MEMBER COUNTRY OF THE EUROPEAN UNION (A) TO DEATH OR PERSONAL INJURY    */
/* RESULTING FROM ETC'S NEGLIGENCE, (B) TO THE VIOLATION OF ESSENTIAL      */
/* CONTRACTUAL DUTIES BY ETC, AND (C) TO DAMAGES RESULTING FROM            */
/* ETC'S GROSS NEGLIGENT OR WILLFUL BEHAVIOR, SOME JURISDICTIONS DO NOT    */
/* ALLOW THE EXCLUSION OR LIMITATION OF INCIDENTAL OR CONSEQUENTIAL        */
/* DAMAGES, SO THIS EXCLUSION AND LIMITATION MAY NOT APPLY TO YOU.         */
/*                                                                         */
/* Neither the name of ETC, its affiliates, nor the names of its           */
/* contributors may be used to endorse or promote products derived from    */
/* this software without express prior written permission of the President */
/* of ETC.                                                                 */
/*                                                                         */
/* The user of this software agrees to indemnify and hold ETC, its         */
/* affiliates, and contributors harmless from and against any and all      */
/* claims, damages, and suits that are in any way connected to or arising  */ 
/* out of, either in whole or in part, this software or the user's use of  */
/* this software.                                                          */
/*                                                                         */
/* Except for the limited license granted herein, all right, title and     */
/* interest to and in the Software remains forever with ETC.               */
/***************************************************************************/

// CID.h: interface for the CID class and DCID class
//
// The CID class encapsulates the functionality for using, passing,
// copying, packing, and comparing SDT CIDs.
// The same goes for the DCID class
//////////////////////////////////////////////////////////////////////

#ifndef _CID_H_
#define _CID_H_

#ifndef _DEFTYPES_H_
#error "#include error, CID.h requires deftypes.h"
#endif

class CID  
{
public:
	enum{
		CIDBYTES = 16,		//The number of bytes in a CID
        // See format string in CIDIntoString().
        // The 16 CID bytes expand into 32 ASCII hexadecimal characters.
        // There are 4 internal dashes, plus 1 terminating null,
        // totaling 37 bytes.
		CIDSTRINGBYTES = 37 //The number of bytes in a CID string, INCLUDING terminating NULL
	};

	CID(const uint1* pCID);  //The CID buffer (in binary form -- Use StringToCID to convert)
	CID(const CID& cid);
	CID();
	virtual ~CID();

	void Pack(uint1* pbuf) const;	//Packs the cid into the buffer -- There must be enough space
	void Unpack(const uint1* pbuf);	//Unpacks the cid from the buffer

	CID& operator=(const CID& cid);

	friend bool operator<(const CID& c1, const CID& c2);
	friend bool operator==(const CID& c1, const CID& c2);
	friend bool operator!=(const CID& c1, const CID& c2);

	//Returns a CID based on the string. 
	static CID StringToCID(const char* ptext);

	//Translates a cid to a preallocated text string of 37 bytes, including the terminating NULL
	static void CIDIntoString(const CID& cid, char* ptxt);

private:
	uint1 m_cid [CIDBYTES];
};

//Essentially the same type as a CID, but for a different semantic purpose
//Just masks the CID type
class DCID 
{
public:
	enum{
		DCIDBYTES = CID::CIDBYTES,				//The number of bytes in a DCID
		DCIDSTRINGBYTES = CID::CIDSTRINGBYTES,	//The number of bytes in a DCID string, INCLUDING terminating NULL
		DCIDFILEBYTES = CID::CIDSTRINGBYTES + 4 //The DCID in the tftp request format: [DCID].ddl, INCLUDING terminating NULL
	};

	DCID(const uint1* pDCID);  //The DCID buffer (in binary form -- Use StringToDCID to convert)
	DCID(const DCID& dcid);
	DCID();
	virtual ~DCID();

	void Pack(uint1* pbuf) const;	//Packs the cid into the buffer -- There must be enough space
	void Unpack(const uint1* pbuf);	//Unpacks the cid from the buffer

	DCID& operator=(const DCID& dcid);

	friend bool operator<(const DCID& d1, const DCID& d2);
	friend bool operator==(const DCID& d1, const DCID& d2);
	friend bool operator!=(const DCID& d1, const DCID& d2);

	//Returns a DCID based on the string. 
	static DCID StringToDCID(const char* ptext);

	//Translates a dcid to a preallocated text string of DCIDSTRINGBYTES bytes, including the terminating NULL
	static void DCIDIntoString(const DCID& dcid, char* ptxt);

	//Translates a dcid as a tftp-able filename to a preallocated text string of DCIDFILEBYTES bytes, including the terminating NULL
	static void DCIDIntoFileName(const DCID& dcid, char* ptxt);

private:
	CID m_cid;
};

bool operator<(const CID& c1, const CID& c2);
bool operator==(const CID& c1, const CID& c2);
bool operator!=(const CID& c1, const CID& c2); 
bool operator<(const DCID& d1, const DCID& d2);
bool operator==(const DCID& d1, const DCID& d2);
bool operator!=(const DCID& d1, const DCID& d2);

#endif // !defined(_CID_H_)

