/***************************************************************************/
/* Copyright (C) 2006 Electronic Theatre Controls, Inc.                    */
/* All rights reserved.                                                    */
/*                                                                         */
/* This software is provided by Electronic Theatre Controls, Inc. ("ETC")  */
/* at no charge to the user to facilitate the use and implementation of    */
/* ANSI BSR E1.31-2006 BSR E1.31, Entertainment Technology - DMX512-A      */
/* Streaming Protocol (DSP).                                               */
/*                                                                         */
/* Redistribution and use in source and binary forms, with or without      */
/* modification, are permitted provided that the following conditions are  */
/* met:                                                                    */
/*                                                                         */
/*      Redistributions of source code, whether in binary form or          */
/*      otherwise must retain the above copyright notice, a list of        */
/*      modifications made, if any, a description of modifications made,   */
/*      this list of conditions and the following disclaimer.              */
/*                                                                         */
/* THIS SOFTWARE IS PROVIDED "AS IS" BY THE COPYRIGHT HOLDER AND WITHOUT   */
/* WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING, WITHOUT     */
/* LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,            */
/* MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.  THE     */
/* ENTIRE RISK AS TO THE QUALITY, OPERATION, AND PERFORMANCE OF THE        */
/* SOFTWARE IS WITH THE USER.                                              */
/*                                                                         */
/* UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY, WHETHER TORT          */
/* (INCLUDING NEGLIGENCE), CONTRACT OR OTHERWISE, SHALL ETC BE LIABLE TO   */
/* ANY PERSON OR ENTITY FOR ANY INDIRECT, SPECIAL, INCIDENTAL, OR          */
/* CONSEQUENTIAL DAMAGES OF ANY KIND INCLUDING WITHOUT LIMITATION, DAMAGES */
/* FOR LOSS OF GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR MALFUNCTION,   */
/* OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF SUCH         */
/* PARTY SHALL HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH DAMAGES.      */
/* THIS LIMITATION OF LIABILITY SHALL NOT APPLY (I) TO DEATH OR PERSONAL   */
/* INJURY RESULTING FROM SUCH PARTY'S NEGLIGENCE TO THE EXTENT APPLICABLE  */
/* LAW PROHIBITS SUCH LIMITATION, (II) FOR CONSUMERS WITH RESIDENCE IN A   */
/* MEMBER COUNTRY OF THE EUROPEAN UNION (A) TO DEATH OR PERSONAL INJURY    */
/* RESULTING FROM ETC'S NEGLIGENCE, (B) TO THE VIOLATION OF ESSENTIAL      */
/* CONTRACTUAL DUTIES BY ETC, AND (C) TO DAMAGES RESULTING FROM            */
/* ETC'S GROSS NEGLIGENT OR WILLFUL BEHAVIOR, SOME JURISDICTIONS DO NOT    */
/* ALLOW THE EXCLUSION OR LIMITATION OF INCIDENTAL OR CONSEQUENTIAL        */
/* DAMAGES, SO THIS EXCLUSION AND LIMITATION MAY NOT APPLY TO YOU.         */
/*                                                                         */
/* Neither the name of ETC, its affiliates, nor the names of its           */
/* contributors may be used to endorse or promote products derived from    */
/* this software without express prior written permission of the President */
/* of ETC.                                                                 */
/*                                                                         */
/* The user of this software agrees to indemnify and hold ETC, its         */
/* affiliates, and contributors harmless from and against any and all      */
/* claims, damages, and suits that are in any way connected to or arising  */ 
/* out of, either in whole or in part, this software or the user's use of  */
/* this software.                                                          */
/*                                                                         */
/* Except for the limited license granted herein, all right, title and     */
/* interest to and in the Software remains forever with ETC.               */
/***************************************************************************/

//VHD.h  
//
// This is a small set of functions that help parse and pack a VHD packet.
// Unfortunately, because the header and data separation is highly protocol
// dependent, there is no nice way to simply have a class that takes care of 
// inheritance, etc.
//

#ifndef _VHD_H_
#define _VHD_H_

#ifndef _DEFTYPES_H_
#error "#include error, VHD.h requires deftypes.h"
#endif

const uint VHD_MAXFLAGBYTES = 7;  //The maximum amount of bytes used to pack the flags, len, and vector
const uint VHD_MAXLEN = 0x0fffff;  //The maximum packet length is 20 bytes long
const uint VHD_MAXMINLENGTH = 4095;  //The highest length that will fit in the "smallest" length pack

//Given a pointer, packs the VHD inheritance flags.  The pointer returned is the 
//SAME pointer, since it is assumed that length will be packed next
uint1* VHD_PackFlags(uint1* pbuffer, bool inheritvec, bool inherithead, bool inheritdata);

//Given a pointer, packs the length.  The pointer returned is the pointer
// to the rest of the buffer.  It is assumed that pbuffer contains at 
// least 3 bytes.  If inclength is true, the resultant length of the flags and
// length field is added to the length parameter before packing
uint1* VHD_PackLength(uint1* pbuffer, uint4 length, bool inclength);

//Given a pointer and vector size, packs the vector.  
//The pointer returned is the pointer to the rest of the buffer.  
//It is assumed that pbuffer contains at least 4 bytes.
uint1* VHD_PackVector(uint1* pbuffer, uint4 vector, uint vecsize);

//Given a pointer, the VHD flags and full PDU length is parsed.  The pointer
// returned is the pointer to the rest of the buffer
const uint1* VHD_GetFlagLength(const uint1* const pbuffer, bool& inheritvec,
								     bool& inherithead, bool& inheritdata, uint4& length);

//Unlike PackVector, which takes a size, it's easier to have 3 different functions that
//do the unpack.  The pointer returned is the pointer to the rest of the buffer
const uint1* VHD_GetVector1(const uint1* const pbuffer, uint1& vector);
const uint1* VHD_GetVector2(const uint1* const pbuffer, uint2& vector);
const uint1* VHD_GetVector4(const uint1* const pbuffer, uint4& vector);

#endif  //_VHD_H_
