/***************************************************************************/
/* Copyright (C) 2006 Electronic Theatre Controls, Inc.                    */
/* All rights reserved.                                                    */
/*                                                                         */
/* This software is provided by Electronic Theatre Controls, Inc. ("ETC")  */
/* at no charge to the user to facilitate the use and implementation of    */
/* ANSI BSR E1.31-2006 BSR E1.31, Entertainment Technology - DMX512-A      */
/* Streaming Protocol (DSP).                                               */
/*                                                                         */
/* Redistribution and use in source and binary forms, with or without      */
/* modification, are permitted provided that the following conditions are  */
/* met:                                                                    */
/*                                                                         */
/*      Redistributions of source code, whether in binary form or          */
/*      otherwise must retain the above copyright notice, a list of        */
/*      modifications made, if any, a description of modifications made,   */
/*      this list of conditions and the following disclaimer.              */
/*                                                                         */
/* THIS SOFTWARE IS PROVIDED "AS IS" BY THE COPYRIGHT HOLDER AND WITHOUT   */
/* WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING, WITHOUT     */
/* LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,            */
/* MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.  THE     */
/* ENTIRE RISK AS TO THE QUALITY, OPERATION, AND PERFORMANCE OF THE        */
/* SOFTWARE IS WITH THE USER.                                              */
/*                                                                         */
/* UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY, WHETHER TORT          */
/* (INCLUDING NEGLIGENCE), CONTRACT OR OTHERWISE, SHALL ETC BE LIABLE TO   */
/* ANY PERSON OR ENTITY FOR ANY INDIRECT, SPECIAL, INCIDENTAL, OR          */
/* CONSEQUENTIAL DAMAGES OF ANY KIND INCLUDING WITHOUT LIMITATION, DAMAGES */
/* FOR LOSS OF GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR MALFUNCTION,   */
/* OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF SUCH         */
/* PARTY SHALL HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH DAMAGES.      */
/* THIS LIMITATION OF LIABILITY SHALL NOT APPLY (I) TO DEATH OR PERSONAL   */
/* INJURY RESULTING FROM SUCH PARTY'S NEGLIGENCE TO THE EXTENT APPLICABLE  */
/* LAW PROHIBITS SUCH LIMITATION, (II) FOR CONSUMERS WITH RESIDENCE IN A   */
/* MEMBER COUNTRY OF THE EUROPEAN UNION (A) TO DEATH OR PERSONAL INJURY    */
/* RESULTING FROM ETC'S NEGLIGENCE, (B) TO THE VIOLATION OF ESSENTIAL      */
/* CONTRACTUAL DUTIES BY ETC, AND (C) TO DAMAGES RESULTING FROM            */
/* ETC'S GROSS NEGLIGENT OR WILLFUL BEHAVIOR, SOME JURISDICTIONS DO NOT    */
/* ALLOW THE EXCLUSION OR LIMITATION OF INCIDENTAL OR CONSEQUENTIAL        */
/* DAMAGES, SO THIS EXCLUSION AND LIMITATION MAY NOT APPLY TO YOU.         */
/*                                                                         */
/* Neither the name of ETC, its affiliates, nor the names of its           */
/* contributors may be used to endorse or promote products derived from    */
/* this software without express prior written permission of the President */
/* of ETC.                                                                 */
/*                                                                         */
/* The user of this software agrees to indemnify and hold ETC, its         */
/* affiliates, and contributors harmless from and against any and all      */
/* claims, damages, and suits that are in any way connected to or arising  */ 
/* out of, either in whole or in part, this software or the user's use of  */
/* this software.                                                          */
/*                                                                         */
/* Except for the limited license granted herein, all right, title and     */
/* interest to and in the Software remains forever with ETC.               */
/***************************************************************************/

/*defpack.h
  This file defines the big and little endian packing/unpacking utilities that
  common code uses.

  All functions follow the form PackXY and UpackXY, 
  where X denotes endianness (B or L, for big and little, respecively),
  and Y denotes the number of bytes (1, 2, 4, 8)

  Windows version follows, 
  which is a little endian machine with no boundary problems.
  
  I've added explicit masking to this version to allow anyone running with "smaller type"
  checking to not have debug exceptions.  
*/
#ifndef	_DEFPACK_H_
#define	_DEFPACK_H_

//defpack requires the default types
#ifndef _DEFTYPES_H_
#error "#include error: defpack.h requires deftypes.h"
#endif

//Declarations
void  PackB1 (uint1* ptr, uint1 val);	//Packs a uint1 to a known big endian buffer
uint1 UpackB1(const uint1* ptr);		//Unpacks a uint1 from a known big endian buffer
void  PackL1 (uint1* ptr, uint1 val);	//Packs a uint1 to a known little endian buffer
uint1 UpackL1(const uint1* ptr);		//Unpacks a uint1 from a known little endian buffer
void  PackB2 (uint1* ptr, uint2 val);	//Packs a uint2 to a known big endian buffer
uint2 UpackB2(const uint1* ptr);		//Unpacks a uint2 from a known big endian buffer
void  PackL2 (uint1* ptr, uint2 val);	//Packs a uint2 to a known little endian buffer
uint2 UpackL2(const uint1* ptr);		//Unpacks a uint2 from a known little endian buffer
void  PackB4 (uint1* ptr, uint4 val);	//Packs a uint4 to a known big endian buffer
uint4 UpackB4(const uint1* ptr);		//Unpacks a uint4 from a known big endian buffer
void  PackL4 (uint1* ptr, uint4 val);	//Packs a uint4 to a known little endian buffer
uint4 UpackL4(const uint1* ptr);		//Unpacks a uint4 from a known little endian buffer
void  PackB8 (uint1* ptr, uint8 val);	//Packs a uint8 to a known big endian buffer
uint8 UpackB8(const uint1* ptr);		//Unpacks a uint4 from a known big endian buffer
void  PackL8 (uint1* ptr, uint8 val);	//Packs a uint8 to a known little endian buffer
uint8 UpackL8(const uint1* ptr);		//Unpacks a uint8 from a known little endian buffer

//---------------------------------------------------------------------------------
//implementation

//Packs a uint1 to a known big endian buffer
inline void	PackB1(uint1* ptr, uint1 val)
{
	*ptr = val;
}

//Unpacks a uint1 from a known big endian buffer
inline uint1 UpackB1(const uint1* ptr)
{
	return *ptr;
}

//Packs a uint1 to a known little endian buffer
inline void	PackL1(uint1* ptr, uint1 val)
{
	*ptr = val;
}

//Unpacks a uint1 from a known little endian buffer
inline uint1 UpackL1(const uint1* ptr)
{
	return *ptr;
}

//Packs a uint2 to a known big endian buffer
inline void PackB2(uint1* ptr, uint2 val)
{
	ptr[1] = (uint1)(val & 0xff);
	ptr[0] = (uint1)((val & 0xff00) >> 8);
}

//Unpacks a uint2 from a known big endian buffer
inline uint2 UpackB2(const uint1* ptr)
{
	return (uint2)(ptr[1] | ptr[0] << 8);
}

//Packs a uint2 to a known little endian buffer
inline void PackL2(uint1* ptr, uint2 val)
{
	*((uint2*)ptr) = val;
}

//Unpacks a uint2 from a known little endian buffer
inline uint2 UpackL2(const uint1* ptr)
{
	return *((uint2*)ptr);
}

//Packs a uint4 to a known big endian buffer
inline void PackB4(uint1* ptr, uint4 val)
{
	ptr[3] = (uint1) (val & 0xff);
	ptr[2] = (uint1)((val & 0xff00) >> 8);
	ptr[1] = (uint1)((val & 0xff0000) >> 16);
	ptr[0] = (uint1)((val & 0xff000000) >> 24);
}

//Unpacks a uint4 from a known big endian buffer
inline uint4 UpackB4(const uint1* ptr)
{
	return (uint4)(ptr[3] | (ptr[2] << 8) | (ptr[1] << 16) | (ptr[0] << 24));
}

//Packs a uint4 to a known little endian buffer
inline void PackL4(uint1* ptr, uint4 val)
{
	*((uint4*)ptr) = val;
}

//Unpacks a uint4 from a known little endian buffer
inline uint4 UpackL4(const uint1* ptr)
{
	return *((uint4*)ptr);
}

//Packs a uint8 to a known big endian buffer
inline void PackB8(uint1* ptr, uint8 val)
{
	ptr[7] = (uint1) (val & 0xff);
	ptr[6] = (uint1)((val & 0xff00) >> 8);
	ptr[5] = (uint1)((val & 0xff0000) >> 16);
	ptr[4] = (uint1)((val & 0xff000000) >> 24);
	ptr[3] = (uint1)((val & 0xff00000000) >> 32);
	ptr[2] = (uint1)((val & 0xff0000000000) >> 40);
	ptr[1] = (uint1)((val & 0xff000000000000) >> 48);
	ptr[0] = (uint1)((val & 0xff00000000000000) >> 56);
}

//Unpacks a uint4 from a known big endian buffer
inline uint8 UpackB8(const uint1* ptr)
{
	return ((uint8)ptr[7]) | (((uint8)ptr[6]) << 8) | (((uint8)ptr[5]) << 16) |
		   (((uint8)ptr[4]) << 24) | (((uint8)ptr[3]) << 32) | 
		   (((uint8)ptr[2]) << 40) | (((uint8)ptr[1]) << 48) | 
		   (((uint8)ptr[0]) << 56);
}

//Packs a uint8 to a known little endian buffer
inline void PackL8(uint1* ptr, uint8 val)
{
	*((uint8*)ptr) = val;
}

//Unpacks a uint8 from a known little endian buffer
inline uint8 UpackL8(const uint1* ptr)
{
	return *((uint8*)ptr);
}


#endif	/*_DEFPACK_H_*/

