/***************************************************************************/
/* Copyright (C) 2006 Electronic Theatre Controls, Inc.                    */
/* All rights reserved.                                                    */
/*                                                                         */
/* This software is provided by Electronic Theatre Controls, Inc. ("ETC")  */
/* at no charge to the user to facilitate the use and implementation of    */
/* ANSI BSR E1.31-2006 BSR E1.31, Entertainment Technology - DMX512-A      */
/* Streaming Protocol (DSP).                                               */
/*                                                                         */
/* Redistribution and use in source and binary forms, with or without      */
/* modification, are permitted provided that the following conditions are  */
/* met:                                                                    */
/*                                                                         */
/*      Redistributions of source code, whether in binary form or          */
/*      otherwise must retain the above copyright notice, a list of        */
/*      modifications made, if any, a description of modifications made,   */
/*      this list of conditions and the following disclaimer.              */
/*                                                                         */
/* THIS SOFTWARE IS PROVIDED "AS IS" BY THE COPYRIGHT HOLDER AND WITHOUT   */
/* WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING, WITHOUT     */
/* LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,            */
/* MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.  THE     */
/* ENTIRE RISK AS TO THE QUALITY, OPERATION, AND PERFORMANCE OF THE        */
/* SOFTWARE IS WITH THE USER.                                              */
/*                                                                         */
/* UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY, WHETHER TORT          */
/* (INCLUDING NEGLIGENCE), CONTRACT OR OTHERWISE, SHALL ETC BE LIABLE TO   */
/* ANY PERSON OR ENTITY FOR ANY INDIRECT, SPECIAL, INCIDENTAL, OR          */
/* CONSEQUENTIAL DAMAGES OF ANY KIND INCLUDING WITHOUT LIMITATION, DAMAGES */
/* FOR LOSS OF GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR MALFUNCTION,   */
/* OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF SUCH         */
/* PARTY SHALL HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH DAMAGES.      */
/* THIS LIMITATION OF LIABILITY SHALL NOT APPLY (I) TO DEATH OR PERSONAL   */
/* INJURY RESULTING FROM SUCH PARTY'S NEGLIGENCE TO THE EXTENT APPLICABLE  */
/* LAW PROHIBITS SUCH LIMITATION, (II) FOR CONSUMERS WITH RESIDENCE IN A   */
/* MEMBER COUNTRY OF THE EUROPEAN UNION (A) TO DEATH OR PERSONAL INJURY    */
/* RESULTING FROM ETC'S NEGLIGENCE, (B) TO THE VIOLATION OF ESSENTIAL      */
/* CONTRACTUAL DUTIES BY ETC, AND (C) TO DAMAGES RESULTING FROM            */
/* ETC'S GROSS NEGLIGENT OR WILLFUL BEHAVIOR, SOME JURISDICTIONS DO NOT    */
/* ALLOW THE EXCLUSION OR LIMITATION OF INCIDENTAL OR CONSEQUENTIAL        */
/* DAMAGES, SO THIS EXCLUSION AND LIMITATION MAY NOT APPLY TO YOU.         */
/*                                                                         */
/* Neither the name of ETC, its affiliates, nor the names of its           */
/* contributors may be used to endorse or promote products derived from    */
/* this software without express prior written permission of the President */
/* of ETC.                                                                 */
/*                                                                         */
/* The user of this software agrees to indemnify and hold ETC, its         */
/* affiliates, and contributors harmless from and against any and all      */
/* claims, damages, and suits that are in any way connected to or arising  */ 
/* out of, either in whole or in part, this software or the user's use of  */
/* this software.                                                          */
/*                                                                         */
/* Except for the limited license granted herein, all right, title and     */
/* interest to and in the Software remains forever with ETC.               */
/***************************************************************************/

/********************************************************************************

  ipaddr.h

  Provides a standard definition of an ip address, useable for both version 4 and 6
  ip address.  

  Incorporates the concept of a network interface.
  
*********************************************************************************/

#ifndef _IPADDR_H_
#define _IPADDR_H_

#ifndef _DEFTYPES_H_
#error "#include error: ipaddr.h requires deftypes.h"
#endif

//The run-time identifier of a NIC interface, used wherever a NIC needs to be identified.  
typedef int netintid;
const netintid NETID_INVALID = -1;

typedef uint2 IPPort;
typedef uint4 IPv4;

class CIPAddr
{
public:
	enum {
		ADDRBYTES = 16,	//The number of bytes in a v6 address
		ADDRSTRINGBYTES =  60  //The maximum number of bytes in a ipaddress string, INCLUDING terminating NULL
	};
	
	CIPAddr();			//The default is an all zero address and port, invalid interface
	CIPAddr(netintid id, IPPort port, IPv4 addr);  //Construct from a port, v4 address, and interface
	CIPAddr(netintid id, IPPort port, const uint1* addr);  //Construct from a port, v6 address, and interface
	CIPAddr(const CIPAddr& addr);
	virtual ~CIPAddr();

	CIPAddr& operator=(const CIPAddr& addr);
	friend bool operator<(const CIPAddr& a1, const CIPAddr& a2);
	friend bool operator==(const CIPAddr& a1, const CIPAddr& a2);
	friend bool operator!=(const CIPAddr& a1, const CIPAddr& a2);

	void SetNetInterface(netintid id);
	netintid GetNetInterface() const;

	void SetIPPort(IPPort port);
	IPPort GetIPPort() const;

	bool IsV4Address() const;
	void SetV4Address(IPv4 addr);
	IPv4 GetV4Address() const;
	void SetV6Address(const uint1* addr);
	const uint1* GetV6Address() const;

	bool IsMulticastAddress() const;

	//Returns an address based on the string, which must be of one of the following forms:
	//d.d.d.d		for ipv4 address
	//d.d.d.d:d		for ipv4 address and port
	//d.d.d.d:d,d	for ipv4 address: port, network interface
	//[x:x:x:x:x:x:x:x]		for ipv6 address
	//[x:x:x:x:x:x:x:x]:d	for ipv6 address and port
	//[x:x:x:x:x:x:x:x]:d,d	for ipv6 address: port, network interface
	static CIPAddr StringToAddr(const char* ptext);

	//Translates an address to a preallocated text string of ADDRSTRINGBYTES, including the terminating NULL
	//Prints it in the forms listed for StringToAddr
	//Note that if showint is true, showport is assumed to be true
	static void AddrIntoString(const CIPAddr& addr, char* ptxt, bool showport, bool showint);


private:
	netintid m_netid;
	IPPort m_port;
	uint1 m_addr [ADDRBYTES];  //Address in big endian format -- upper bytes all 0's for ipv4
};

bool operator<(const CIPAddr& a1, const CIPAddr& a2);
bool operator==(const CIPAddr& a1, const CIPAddr& a2);
bool operator!=(const CIPAddr& a1, const CIPAddr& a2);

#endif


